@@ -1,7 +1,7 @@
 # function plots reconstructed values for ancestral characters along the edges of the tree
 # written by Liam J. Revell 2012-2013
 
-contMap<-function(tree,x,res=100,fsize=NULL,ftype=NULL,lwd=4,legend=NULL,lims=NULL,outline=TRUE,sig=3,type="phylogram",plot=TRUE,...){
+contMap<-function(tree,x,res=100,fsize=NULL,ftype=NULL,lwd=4,legend=NULL,lims=NULL,outline=TRUE,sig=3,type="phylogram",direction="rightwards",plot=TRUE,...){
 	if(hasArg(mar)) mar<-list(...)$mar
 	else mar<-rep(0.3,4)
 	h<-max(nodeHeights(tree))
@@ -28,7 +28,7 @@ contMap<-function(tree,x,res=100,fsize=NULL,ftype=NULL,lwd=4,legend=NULL,lims=NU
 	tree$mapped.edge<-tree$mapped.edge[,order(as.numeric(colnames(tree$mapped.edge)))]
 	xx<-list(tree=tree,cols=cols,lims=lims)
 	class(xx)<-"contMap"
-	if(plot) plot.contMap(xx,fsize=fsize,ftype=ftype,lwd=lwd,legend=legend,outline=outline,sig=sig,type=type,mar=mar)
+	if(plot) plot.contMap(xx,fsize=fsize,ftype=ftype,lwd=lwd,legend=legend,outline=outline,sig=sig,type=type,mar=mar,direction=direction)
 	invisible(xx)
 }
 
@@ -71,6 +71,8 @@ plot.contMap<-function(x,...){
 	else type<-"phylogram"
 	if(hasArg(mar)) mar<-list(...)$mar
 	else mar<-rep(0.3,4)
+	if(hasArg(direction)) direction<-list(...)$direction
+	else direction<-"rightwards"
 	if(is.null(legend)) legend<-0.5*max(H)
 	if(is.null(fsize)) fsize<-c(1,1)
 	if(length(fsize)==1) fsize<-rep(fsize,2)
@@ -78,6 +80,6 @@ plot.contMap<-function(x,...){
 	if(length(ftype)==1) ftype<-c(ftype,"reg")
 	# done optional arguments
 	leg.txt<-c(round(lims[1],sig),"trait value",round(lims[2],sig))
-	plot(x,fsize=fsize,ftype=ftype,lwd=lwd,legend=legend,outline=outline,leg.txt=leg.txt,type=type,mar=mar)
+	plot(x,fsize=fsize,ftype=ftype,lwd=lwd,legend=legend,outline=outline,leg.txt=leg.txt,type=type,mar=mar,direction=direction)
 }
 
